@@ -10,7 +10,7 @@ library to build dynamic html that provides clean separation of form
 making cross-site scripting attacks and improperly generated html
 very difficult.
 
-Copyright (c) 2003-2006 Ian Ward
+Copyright (c) 2003-2009 Ian Ward
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -31,6 +31,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 from __future__ import nested_scopes
 
 import sys
+import os
 
 import templayer
 
@@ -82,9 +83,9 @@ def example_lawn2():
 	sys.stdout.write("Content-type: text/html\r\n\r\n")
 
 	tmpl = templayer.HTMLTemplate("lawn2.html")
-	file_layer = tmpl.start_file()
-	file_layer.open()
-	file_layer.close()
+	file_writer = tmpl.start_file()
+	file_writer.open()
+	file_writer.close()
 
 template["lawn3"] = """
 <html>
@@ -115,10 +116,10 @@ def example_lawn3():
 	sys.stdout.write("Content-type: text/html\r\n\r\n")
 
 	tmpl = templayer.HTMLTemplate("lawn3.html")
-	file_layer = tmpl.start_file()
-	file_layer.open(title="Gordon's Lawn Happenings",
+	file_writer = tmpl.start_file()
+	file_writer.open(title="Gordon's Lawn Happenings",
 			date=time.asctime())
-	file_layer.close()
+	file_writer.close()
 
 template["lawn4"] = """
 <html>
@@ -148,15 +149,15 @@ def example_lawn4a():
 	sys.stdout.write("Content-type: text/html\r\n\r\n")
 
 	tmpl = templayer.HTMLTemplate("lawn4.html")
-	file_layer = tmpl.start_file()
-	main_layer = file_layer.open(title="Gordon's Lawn Happenings",
+	file_writer = tmpl.start_file()
+	main_layer = file_writer.open(title="Gordon's Lawn Happenings",
 		date=time.asctime())
 	main_layer.write_layer('report', day="Sunday", happenings=[
 		"We've got a groundhog.  I will have to stay alert.",
 		"I lost half a tomato plant to that furry guy."])
 	main_layer.write_layer('report', day="Monday", happenings=[
 		"The grass grew - I saw it."])
-	file_layer.close()
+	file_writer.close()
 
 def example_lawn4b():
 	import templayer
@@ -172,12 +173,12 @@ def example_lawn4b():
 	]
 
 	tmpl = templayer.HTMLTemplate("lawn4.html")
-	file_layer = tmpl.start_file()
-	main_layer = file_layer.open(title="Gordon's Lawn Happenings",
+	file_writer = tmpl.start_file()
+	main_layer = file_writer.open(title="Gordon's Lawn Happenings",
 		date=time.asctime())
 	for d, h in reports:
 		main_layer.write_layer('report', day=d, happenings=h)
-	file_layer.close()
+	file_writer.close()
 
 def example_lawn4c():
 	import templayer
@@ -193,12 +194,12 @@ def example_lawn4c():
 	]
 
 	tmpl = templayer.HTMLTemplate("lawn4.html")
-	file_layer = tmpl.start_file()
-	main_layer = file_layer.open(title="Gordon's Lawn Happenings",
+	file_writer = tmpl.start_file()
+	main_layer = file_writer.open(title="Gordon's Lawn Happenings",
 		date=time.asctime())
 	for d, h in reports:
 		main_layer.write_layer('report', day=d, happenings=h)
-	file_layer.close()
+	file_writer.close()
 
 template["lawn5"] = """
 <html>
@@ -237,8 +238,8 @@ def example_lawn5():
 	]
 
 	tmpl = templayer.HTMLTemplate("lawn5.html")
-	file_layer = tmpl.start_file()
-	main_layer = file_layer.open(title="Gordon's Lawn Happenings",
+	file_writer = tmpl.start_file()
+	main_layer = file_writer.open(title="Gordon's Lawn Happenings",
 		date=time.asctime())
 	for d, h in reports:
 		happening_list = []
@@ -246,7 +247,7 @@ def example_lawn5():
 			formatted = tmpl.format('happening', what=w)
 			happening_list.append(formatted)
 		main_layer.write_layer('report', day=d, happenings=happening_list)
-	file_layer.close()
+	file_writer.close()
 
 template["lawn6"] = """
 <html>
@@ -285,14 +286,14 @@ def example_lawn6a():
 	]
 
 	tmpl = templayer.HTMLTemplate("lawn6.html")
-	file_layer = tmpl.start_file()
-	main_layer = file_layer.open(title="Gordon's Lawn Happenings",
+	file_writer = tmpl.start_file()
+	main_layer = file_writer.open(title="Gordon's Lawn Happenings",
 		date=time.asctime())
 	for d, h in reports:
 		report_layer = main_layer.open_layer('report', day=d)
 		for happening in h:
 			report_layer.write_layer('happening', contents=happening)
-	file_layer.close()
+	file_writer.close()
 
 def example_lawn6b():
 	import templayer
@@ -308,15 +309,15 @@ def example_lawn6b():
 	]
 
 	tmpl = templayer.HTMLTemplate("lawn6.html")
-	file_layer = tmpl.start_file()
-	main_layer = file_layer.open(title="Gordon's Lawn Happenings",
+	file_writer = tmpl.start_file()
+	main_layer = file_writer.open(title="Gordon's Lawn Happenings",
 		date=time.asctime())
 	for d, h in reports:
 		report_layer = main_layer.open_layer('report', day=d)
 		for happening in h:
 			happening_layer = report_layer.open_layer('happening')
 			happening_layer.write(happening)
-	file_layer.close()
+	file_writer.close()
 
 def example_lawn6c():
 	import templayer
@@ -332,8 +333,8 @@ def example_lawn6c():
 	]
 
 	tmpl = templayer.HTMLTemplate("lawn6.html")
-	file_layer = tmpl.start_file()
-	main_layer = file_layer.open(title="Gordon's Lawn Happenings",
+	file_writer = tmpl.start_file()
+	main_layer = file_writer.open(title="Gordon's Lawn Happenings",
 		date=time.asctime())
 	for d, h in reports:
 		report_layer = main_layer.open_layer('report', day=d)
@@ -341,87 +342,158 @@ def example_lawn6c():
 			happening_layer = report_layer.open_layer('happening')
 			happening_layer.write(happening)
 		main_layer.close_child()
-		file_layer.flush()
-	file_layer.close()
+		file_writer.flush()
+	file_writer.close()
 
 
-# NOT CURRENTLY USED
-#template["guest1"] = 
-"""
+
+
+
+template["simple.templayer"] = """
 <html>
 <head><title>%title%</title></head>
 <body>
-<h1>%title%</h1>
-Sign my guest book!<br/>
-<form method="post">
-Your Name: <input type="text" name="guest"/><br/>
-Comments:<br/>
-<textarea name="comments" rows="5" cols="40"></textarea><br/>
-<input type="submit" name="submit" value="Sign Guest Book"/>
-</form>
-<hr/>
-{contents}
 
-{guest}
-<p>
-<u>%date%: <b>%guest%</b></u><br/>
-%contents%
-</p>
-{/guest}
+<h1>%title%</h1>
+{contents}
 
 {/contents}
 </body>
 </html>
-
 """
 
-# NOT CURRENTLY USED
-examples["guest1"] = ["example_guest1"]
-def example_guest1():
+examples["simple.templayer"] = ["example_simple_views_1", "example_simple_views_2"]
+def example_simple_views_1():
+	from django.http import HttpResponse
+	import datetime
 	import templayer
-	import time
-	import cgi
-	import os
-	import sys
 
-	guest_dir = "guests"
-	max_name = 30
-	max_comment = 300
+	tmpl = templayer.get_django_template("simple.templayer.html")
 
-	cgi_d = cgi.parse()
-	if cgi_d.has_key("guest"):
-		# Add the Guest to my Guest Book
-		timestamp = "%013.02f" % time.time()
-		comments = ""
-		if cgi_d.has_key("comments"):
-			comments = cgi_d["comments"][0][:max_comment]
-		guest = cgi_d["guest"][0][:max_name]
-		f = open(os.path.join(guest_dir, timestamp), "w")
-		f.write(guest + "\n" + comments)
-		f.close()
-		
-	if os.environ['REQUEST_METHOD']=='POST':
-		# It's always a good idea to redirect after a POST request
-		sys.stdout.write("Location: http://" +
-			 os.environ['SERVER_NAME'] +
-			 os.environ['REQUEST_URI'] + "\r\n\r\n" )
-		sys.exit(0)
-	
-	sys.stdout.write("Content-type: text/html\r\n\r\n")
+	def current_datetime(request):
+		file_writer = tmpl.start_file(HttpResponse())
+		now = datetime.datetime.now()
+		contents = file_writer.open(title="Current Date and Time")
+		contents.write("It is now %s." % now)
+		return file_writer.close()
 
-	tmpl = templayer.HTMLTemplate("guest1.html")
-	file_layer = tmpl.start_file()
-	main_layer = file_layer.open(title="Gordon's Guest Book")
+def example_simple_views_2():
+	import datetime
+	import templayer
+
+	tmpl = templayer.get_django_template("simple.templayer.html")
+
+	@templayer.django_view(tmpl)
+	def current_datetime(file_writer, request):
+		now = datetime.datetime.now()
+		contents = file_writer.open(title="Current Date and Time")
+		contents.write("It is now %s." % now)
+
+template["book.templayer"] = """
+<html>
+<head><title>%title%</title></head>
+<body>
+
+<h1>%title%</h1>
+{contents}
+
+{new_entry}
+<form method="POST">
+<h2>Sign the guestbook</h2>
+Your Name: %form.name% %form.name.errors%<br/>
+Rate the site: %form.rating% %form.rating.errors%<br/>
+Your Comments:<br/>
+%form.comment% %form.comment.errors%<br/>
+<input type="submit" value="Submit"/>
+</form>
+{/new_entry}
+
+{/contents}
+</body>
+</html>
+"""
+
+examples["book.templayer"] = ["example_book_models", "example_book_views"]
+def example_book_models():
+	from django.db import models
+
+	RATING_CHOICES = (
+		('G', "Great!"),
+		('A', "Average"),
+		('U', "Uninteresting"),
+	)
+
+	class Entry(models.Model):
+		name = models.CharField("Your Name", max_length=100)
+		rating = models.CharField("Rate this site", max_length=1,
+			choices=RATING_CHOICES)
+		comment = models.TextField("Your Comments (optional)", blank=True)
+		posted = models.DateTimeField(auto_now_add=True)
+
+def example_book_views():
+	from book.models import Entry
+	from django.forms import ModelForm
+
+	import templayer
+
+	tmpl = templayer.get_django_template("book.templayer.html")
+
+	class EntryForm(ModelForm):
+		class Meta:
+			model = Entry
+
+	@templayer.django_view(tmpl)
+	def guest_book(file_layer, request):
+		if request.POST:
+			entry_form = EntryForm(request.POST)
+			# TODO: do something with the data if all is well
+		else:
+			entry_form = EntryForm()
+		contents = file_writer.open(title="Guest Book")
+		contents.write_layer("new_entry", **templayer.django_form(entry_form))
+
+template["emulate.templayer"] = """
+<html>
+<head><title>%title%</title></head>
+<body>
+
+<h1>%title%</h1>
+{contents}
+
+{show_404}
+The url %url% could not be found.
+{/show_404}
+
+{flatpage_body}
+%contents%
+{/flatpage_body}
+
+{/contents}
+</body>
+</html>
+"""
+
+examples["emulate.templayer"] = ["example_templayer_pages"]
+def example_templayer_pages():
+	import templayer
+
+	tmpl = templayer.get_django_template("emulate.templayer.html")
+
+	@templayer.django_template(tmpl, "404.html")
+	def show_404(file_writer, context, request_path):
+		contents = file_writer.open(title="Page not found")
+
+		# request_path == context['request_path']
+		contents.write_layer("show_404", url=request_path)
+
+	@templayer.django_template(tmpl, "flatpages/default.html")
+	def show_flatpage(file_writer, context, flatpage):
+		contents = file_writer.open(title=flatpage.title)
+		fp_contents = contents.open_layer("flatpage_body")
+
+		# the flatpage content is HTML, so don't escape it.
+		fp_contents.write(templayer.RawHTML(flatpage.content))
 
-	timestamps = os.listdir(guest_dir)
-	timestamps.sort()
-	for ts in timestamps:
-		f = open(os.path.join(guest_dir, ts))
-		guest, comment = f.read().split("\n", 1)
-		date = time.asctime(time.localtime(float(ts)))
-		main_layer.write_layer('guest', guest=guest, date=date,
-			contents=comment)
-	file_layer.close()
 
 
 
@@ -493,7 +565,7 @@ def write_example_files(sections, blocks):
 		open(t+".html", "w").write(template[t].lstrip())
 	
 			
-class SimulatedOutput:
+class SimulatedOutput(object):
 	output = []
 	def write(self, s):
 		self.output.append(s)
@@ -501,7 +573,7 @@ class SimulatedOutput:
 	def flush(self):
 		pass
 
-class NullOutput:
+class NullOutput(object):
 	def write(self, s):
 		pass
 
@@ -519,7 +591,8 @@ class SimulatedHTMLTemplate(templayer.HTMLTemplate):
 		head, t = t.split("<body>",1)
 		t, tail = t.split("</body>",1)
 		t = t.strip()
-		self.set_template_data(template[name])
+		self.encoding = "utf_8"
+		self.set_template_data(template[name].decode(self.encoding))
 	
 	def start_file(self, file=None):
 		return templayer.FileLayer(SimulatedOutput(), self)
@@ -609,7 +682,7 @@ def generate_body(tmpl, sections, blocks):
 			b = tmpl.format('body[%s]'%tag, ** slots )
 			b = tmpl.format('section_body', snum=`snum`,
 				inum=`inum`, name=name, tag=tag,
-				content = b)
+				contents = b)
 			body.append( b )
 			
 	return (body, toc_slots)
@@ -647,15 +720,9 @@ def parse_options():
 def main():
 	(do_html, do_scripts) = parse_options()
 
-	if templayer is None:
-		sys.stderr.write(
-"""Error importing templayer.  Please download and install the Templayer
-python module available at:
-http://excess.org/templayer/
-""")
-		sys.exit( 1 )
-
-	tmpl = templayer.HTMLTemplate( "tmpl_tutorial.html" )
+	tmpl = templayer.HTMLTemplate(
+		os.path.join(os.path.dirname(sys.argv[0]),
+		"tmpl_tutorial.html"))
 	sections = read_sections( tmpl )
 	code_blocks = read_example_code()
 
@@ -666,7 +733,8 @@ http://excess.org/templayer/
 		out_file = tmpl.start_file()
 		(body, toc_slots) = generate_body( tmpl, sections,
 		                                   code_blocks )
-		bottom = out_file.open( ** toc_slots )
+		bottom = out_file.open(version=templayer.__version__,
+			** toc_slots)
 		bottom.write( body )
 		out_file.close()
 
