@@ -10,7 +10,7 @@ library to build dynamic html that provides clean separation of form
 making cross-site scripting attacks and improperly generated html
 very difficult.
 
-Copyright (c) 2003-2006 Ian Ward
+Copyright (c) 2003-2009 Ian Ward
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -29,13 +29,15 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 """
 
 __author__ = "Ian Ward"
-__version__ = "1.4"
+__version__ = "1.5.1"
 
 import string
 import sys
 import os
 import time
 import re
+from types import TupleType, ListType, IntType, UnicodeType
+from inspect import getargspec
 
 try: True # old python?
 except: False, True = 0, 1
@@ -51,22 +53,24 @@ COMPLETE_FILE_LAYER = "*"
 
 ENTITY_RE = re.compile("^#x[0-9A-Fa-f]+$|^#[0-9]+$|^[a-zA-Z]+$")
 
+TEMPLAYER_PAGES_MODULE = "templayer_pages"
+
 class TemplateError(Exception):
 	pass
 
-class Template:
+class Template(object):
 	"""
 	class Template - common functionality for Template subclasses
 	"""
-	def __init__( self, name, from_string=False, auto_reload=None,
-		allow_degenerate=False ):
+	def __init__( self, name, from_string=False, auto_reload='never',
+		allow_degenerate=False, encoding='utf_8'):
 		"""
 		name -- template file name or file-like object
 		from_string -- if True treat name as a string containing the
 		               template data instead of a file to read from
 		auto_reload -- if the template has not been checked in this
 		               many seconds then check it and reload if there
-			       are any changes.  If None only load the 
+			       are any changes.  If 'never' only load the 
 			       template file once.
 
 			       Checking and reloading is done by the 
@@ -78,41 +82,54 @@ class Template:
 		               have a {contents}...{/contents} block.  In this
 			       case the entire file is available as a layer
 			       called '*'.  eg. l = tmpl.format('*', ...)
+		encoding -- encoding for template file
 		"""
 		
 		self.allow_degenerate = allow_degenerate
+		self.encoding = encoding
+		self.auto_reload = auto_reload
+		self.template_ts = 0
 
+		self.template_name = None
 		if from_string:
-			self.set_template_data( name )
+			data = name
+			if type(data)!=UnicodeType:
+				data = data.decode(encoding)
+			self.set_template_data(data)
 			auto_reload = None
 		elif hasattr(name, "read"):
-			self.set_template_data( name.read() )
-			auto_reload = None
+			self.set_template_data(name.read().decode(encoding))
+			auto_reload = 'never'
 		else:
 			self.template_name = name
 			self.reload_template()
 
-		self.auto_reload = auto_reload
 		
 	
 	def reload_template(self):
 		"""Reload the template data from the template file."""
-		tname = self.template_name
-		self.template_mtime = os.stat(tname)[8]
-		tmpl = open(tname).read()
+		if not self.template_name: return
+		tmpl = open(self.template_name).read().decode(self.encoding)
 		self.set_template_data(tmpl)
-		self.template_ts = time.time()
 
 	def template_file_changed(self):
 		"""
 		Return True if the template file has been modified since
 		the last time it was loaded.
 		"""
+		if self.template_ts == "never":
+			return False
+		if not self.template_name:
+			return True # we don't know if it has changed
 		if os.stat(self.template_name)[8] != self.template_ts:
 			return True
 
 	def set_template_data(self,tmpl):
 		"""Parse and store the template data passed."""
+		if type(tmpl) != UnicodeType:
+			raise TemplateError("template data must be "
+				"a unicode string")
+
 		self.cache = {}
 		try:
 			# use the first CONTENTS_BEGIN in the file
@@ -239,7 +256,11 @@ class Template:
 
 
 	def format( self, layer_name, ** args ):
-		"""Return a layer with slots filled by args."""
+		"""
+		Return a layer with slots filled by args.
+		
+		self.format(...) and self(...) are equivalent.
+		"""
 		s = self.layer_split(layer_name)
 
 		fargs = {}
@@ -253,10 +274,12 @@ class Template:
 			self.missing_slot(missing, layer_name)
 		
 		return self.post_process(s)
+	
+	__call__ = format
 
-	def start_file( self, file = None ):
+	def start_file( self, file=None, encoding='utf_8' ):
 		"""
-		Return a FileLayer object that uses this template.
+		Return a FileWriter object that uses this template.
 
 		file -- file object or None to use sys.stdout.
 		
@@ -265,13 +288,13 @@ class Template:
 		"""
 		if self.auto_reload is not None:
 			t = time.time()
-			if t > self.template_ts+self.auto_reload:
+			if self.auto_reload != "never" and \
+				t > self.template_ts+self.auto_reload:
 				if self.template_file_changed():
 					self.reload_template()
-				else:
-					self.template_ts = t
+				self.template_ts = t
 		
-		return FileLayer( file, self )
+		return FileWriter( file, self, encoding )
 
 	def pre_process( self, key, value ):
 		"""
@@ -280,7 +303,6 @@ class Template:
 		Override this function to provide escaping or filtering of
 		text sent from the module user.
 		"""
-		assert type(value) == type("")
 		return key, value
 	
 	def post_process( self, value ):
@@ -312,7 +334,7 @@ def fill_slots(layer_split, **args):
 			last_is_slot = False
 			o.append(p)
 		elif args.has_key(p):
-			o.append(args[p])
+			o.append(unicode(args[p]))
 			filled[p] = True
 			last_is_slot = True
 		else:
@@ -350,10 +372,13 @@ class HTMLTemplate(Template):
 		"""
 		assert isinstance(value,RawHTML)
 		return value.value
+
 # backwards compatibility
 HtmlTemplate = HTMLTemplate
 
 
+
+
 class MarkupError(Exception):
 	pass
 # backwards compatibility
@@ -376,6 +401,42 @@ MKE_UNRECOGNISED = "tuple must begin with one of the following strings: " \
 	"'join', 'urljoin', 'href', 'target', 'br', 'p', 'i', 'b'"
 
 
+def join(markup_list, separator):
+	"""HTML Markup like markup_list.join(separator)"""
+	return expand_html_markup(('join', markup_list, separator))
+def pluralize(count, sigular_content, plural_content):
+	"""HTML Markup (singular_content if count==1 else plural_content)"""
+	return expand_html_markup(
+		('pluralize', count, singular_content, plural_content))
+def urljoin(head, tail):
+	"""HTML Markup join safe url head with unsafe tail"""
+	return expand_html_markup(('urljoin', head, tail))
+def href(link, content):
+	"""HTML Markup <a href="link">content</a>"""
+	return expand_html_markup(('href', link, content))
+def target(name):
+	"""HTML Markup <a name="name"></a>"""
+	return expand_html_markup(('target', name))
+def BR(count=1):
+	"""HTML Markup <br/> * count"""
+	return expand_html_markup(('br', count))
+def P(content=""):
+	"""HTML Markup <p>content</p>"""
+	return expand_html_markip(('p', content))
+def I(content):
+	"""HTML Markup <i>content</i>"""
+	return expand_html_markip(('i', content))
+def B(content):
+	"""HTML Markup <b>content</b>"""
+	return expand_html_markip(('b', content))
+def U(content):
+	"""HTML Markup <u>content</u>"""
+	return expand_html_markip(('u', content))
+def entity(entity):
+	"""HTML Markup &entity;"""
+	return expand_html_markip(('&', entity))
+
+
 def expand_html_markup( v ):
 	"""
 	Return an HTML string based on markup v.
@@ -406,30 +467,28 @@ def expand_html_markup( v ):
 	if isinstance(v,RawHTML):
 		return v.value
 		
-	if type(v) == type("") or type(v) == type(u""): 
-		return html_escape(v)
-		
-	if type(v) == type([]):
+	if type(v) == ListType:
 		l = []
 		for x in v:
 			l.append( expand_html_markup( x ) )
 		return "".join(l)
 
-	if type(v) != type(()):
-		raise MarkupError(MKE_BAD_TYPE % `v`)
+	if type(v) != TupleType:
+		try:
+			return html_escape(v)
+		except AttributeError, err:
+			raise MarkupError(MKE_BAD_TYPE % repr(type(v)))
 	
 	if v[0] == 'href':
 		if len(v)!=3:
 			raise MarkupError(MKE_INVALID_HREF)
-		if type(v[1]) == type(""):
+		if type(v[1]) != TupleType or v[1][0]!='urljoin':
 			v=[ v[0], ('urljoin',v[1],"") ,v[2] ]
-		if type(v[1])!=type(()) or v[1][0]!='urljoin':
-			raise MarkupError(MKE_INVALID_HREF)
 		return html_href(expand_html_markup(v[1]),
 			expand_html_markup(v[2]))
 		
 	if v[0] == 'join':
-		if len(v)!=3 or type(v[1]) != type([]):
+		if len(v)!=3 or type(v[1]) != ListType:
 			raise MarkupError(MKE_INVALID_JOIN)
 		sep = expand_html_markup(v[2])
 		l = []
@@ -446,23 +505,28 @@ def expand_html_markup( v ):
 			return expand_html_markup(v[3])
 		
 	if v[0] == 'urljoin':
-		if len(v)!=3 or type(v[1])!=type("") or type(v[2])!=type(""):
+		if len(v)!=3:
+			raise MarkupError(MKE_INVALID_URLJOIN)
+		try:
+			return v[1]+html_url_encode(v[2])
+		except TypeError, err:
+			raise MarkupError(MKE_INVALID_URLJOIN)
+		except AttributeError, err:
 			raise MarkupError(MKE_INVALID_URLJOIN)
-		return v[1]+html_url_encode(v[2])
 	
 	if v[0] == 'target':
-		if len(v)!=3 or type(v[1])!=type(""):
+		if len(v)!=3:
 			raise MarkupError(MKE_INVALID_TARGET)
 		return html_target(html_url_encode(v[1]),
 			expand_html_markup(v[2]))
 			
-	if v[0] == 'p' and len(v) == 2 and type(v[1])!=type(0):
+	if v[0] == 'p' and len(v) == 2 and type(v[1]) != IntType:
 		return "<p>"+expand_html_markup(v[1])+"</p>"
 		
 	if v[0] == 'br' or v[0] == 'p':
 		if len(v)==1:
 			v = (v[0], 1)
-		if len(v)!=2 or type(v[1])!=type(0):
+		if len(v)!=2 or type(v[1]) != IntType:
 			raise MarkupError(MKE_INVALID_BR_P %(v[0],v[0]))
 		return ("<"+v[0]+"/>") * v[1]
 		
@@ -474,7 +538,7 @@ def expand_html_markup( v ):
 			"</"+v[0]+">")
 	
 	if v[0] == '&':
-		if len(v)!=2 or type(v[1])!=type(""):
+		if len(v)!=2:
 			raise MarkupError(MKE_INVALID_ENTITY)
 		if not ENTITY_RE.match(v[1]):
 			raise MarkupError(MKE_INVALID_ENTITY)
@@ -483,7 +547,7 @@ def expand_html_markup( v ):
 	raise MarkupError(MKE_UNRECOGNISED)
 
 
-class RawHTML:
+class RawHTML(object):
 	"""
 	class RawHTML - wrap strings of generated html that are passed 
 	outside the module so that they aren't escaped when passed back in
@@ -492,29 +556,30 @@ class RawHTML:
 		"""value -- segment of HTML as a string"""
 		self.value = value
 	def __repr__( self ):
-		return 'RawHTML(%s)'%`self.value`
+		return 'RawHTML(%s)'%repr(self.value)
 
 
 class LayerError(Exception):
 	pass
 
-class FileLayer:
+class FileWriter(object):
 	"""
-	class FileLayer - the layer within which all other layers nest,
+	class FileWriter - the layer within which all other layers nest,
 	responsible for sending generated text to the output stream
 	"""
-	def __init__( self, file, template ):
+	def __init__( self, file, template, encoding='utf_8' ):
 		"""
 		file -- output stream or None to use sys.stdout
 		template -- template object for generating this file
 
 		This constructor is usually not called directly.  Use
-		template.start_file() to create a FileLayer instead.
+		template.start_file() to create a FileWriter instead.
 		"""
 		if not file: file = sys.stdout
 		self.out = file
 		self.template = template
 		self.child = None
+		self.encoding = encoding
 	
 	def open( self, ** args ):
 		"""
@@ -532,8 +597,9 @@ class FileLayer:
 		"""
 		Flush as much output as possible to the output stream.
 		"""
-		assert self.child != None, "FileLayer already closed!"
-		self.out.write( "".join(self.child.flush()) )
+		assert self.child != None, "FileWriter already closed!"
+		data ="".join(self.child.flush())
+		self.out.write(data.encode(self.encoding))
 		self.out.flush()
 	
 	def close( self ):
@@ -544,13 +610,18 @@ class FileLayer:
 
 		Returns the file object where output was sent.
 		"""
-		assert self.child != None, "FileLayer already closed!"
-		self.out.write( "".join(self.child.close()) )
+		assert self.child != None, "FileWriter.close() already called "\
+			"or .open() was never called!"
+		data ="".join(self.child.close())
+		self.out.write(data.encode(self.encoding))
 		self.child = None
 		return self.out
-	
 
-class Layer:
+# backwards compatibility	
+FileLayer = FileWriter
+
+
+class Layer(object):
 	"""
 	class Layer - holds the state of one of the nested layers while its
 	contents are being written to.  Layers are closed implicitly when
@@ -564,7 +635,7 @@ class Layer:
 		footer -- text after the content of this layer
 
 		This constructor should not be called directly.  Use
-		the open() function in a FileLayer or the open_layer()
+		the open() function in a FileWriter or the open_layer()
 		function in another Layer object to create a new Layer.
 		"""
 		self.child = None
@@ -639,7 +710,7 @@ class Layer:
 		Flush as much output as possible and return it.
 
 		This function should not be called directly.  Use the
-		flush() function in the FileLayer object instead.
+		flush() function in the FileWriter object instead.
 		"""
 		if self.out == None:
 			raise LayerError, "Layer is closed!"
@@ -654,7 +725,7 @@ class Layer:
 		Close this layer and return its output.
 		
 		This function should not be called directly.  Use the
-		close() function in the FileLayer object instead.
+		close() function in the FileWriter object instead.
 		"""
 		self.close_child()
 		final = self.out+[self.footer]
@@ -703,10 +774,10 @@ def html_escape(text):
 	text -- text to be escaped for inclusion within HTML.
 	"""
 
-	text = string.replace(text,'&','&amp;')
-	text = string.replace(text,'"','&quot;') # in case we're in an attrib.
-	text = string.replace(text,'<','&lt;')
-	text = string.replace(text,'>','&gt;')
+	text = text.replace('&','&amp;')
+	text = text.replace('"','&quot;') # in case we're in an attrib.
+	text = text.replace('<','&lt;')
+	text = text.replace('>','&gt;')
 	return text
 
 
@@ -734,18 +805,217 @@ def django_form(form, errors=True, join_errors=lambda x: ('join', x, ('br,'))):
 	tmpl.format('form_layer', title="hello", **django_form(form))
 	"""
 	d = {}
-	for f in form.fields:
-		value = form[f.field_name]
-		d["form."+f.field_name] = RawHTML(str(value))
+	for field_name in form.fields:
+		value = form[field_name]
+		d["form."+field_name] = RawHTML(str(value))
 		if errors:
-			e = [str(x) for x in value.errors()]
-			d["form." + f.field_name + ".errors"] = join_errors(e)
+			e = [x for x in form.errors.get(field_name,[])]
+			d["form." + field_name + ".errors"] = join_errors(e)
 	return d
 
 
-def main():
-	import sys
-	print sys.argv
+
+class DjangoTemplayerError(Exception):
+	pass
+
+class _DjangoTemplayer(object):
+	def __init__(self, run, template_name, tmpl, args, optargs):
+		self.run = run
+		self.template_name = template_name
+		self.tmpl = tmpl
+		self.args = args
+		self.optargs = optargs
+
+	def render(self, context_instance):
+		from django.http import HttpResponse
+
+		sendargs = {}
+		for arg in self.args:
+			if arg not in context_instance:
+				raise DjangoTemplayerError("Required "
+					"parameter %s was not passed to "
+					"template %s!" % (
+					repr(arg), repr(self.template_name)))
+			sendargs[arg] = context_instance[arg]
+		for arg in self.optargs:
+			if arg in context_instance:
+				sendargs[arg] = context_instance[arg]
+
+		# create the file layer before calling our function
+		fwriter = self.tmpl.start_file(HttpResponse())
+		response = self.run(fwriter, context_instance, **sendargs)
+		# if the file layer (or None) is returned, render it
+		if response is fwriter or response is None:
+			response = fwriter.close()
+		return response.content
+
+
+_django_templates = {}
+_django_templates_searched = False
+
+def django_template_loader(template_name, template_dirs=None):
+	"""
+	This function should be included in settings.TEMPLATE_LOADERS when
+	using the django_template decorator.
+
+	eg:
+	TEMPLATE_LOADERS += ['templayer.django_template_loader']
+	"""
+	global _django_templates
+	global _django_templates_searched
+
+	from django.template import TemplateDoesNotExist
+
+	if template_name not in _django_templates and \
+		not _django_templates_searched:
+		_django_templates_searched = True
+		# django template registration is a side-effect of importing
+		# a module with django_template decorators, so try
+		# looking in the views modules of installed applications
+		
+		from django.conf import settings
+		targets = []
+		for droot in settings.TEMPLATE_DIRS:
+			for d, subdirs, files in os.walk(droot):
+				if TEMPLAYER_PAGES_MODULE+".py" in files:
+					targets.append(d)
+
+		for t in targets:
+			orig_sys_path = sys.path
+			try:
+				sys.path = [t] + orig_sys_path
+				__import__(TEMPLAYER_PAGES_MODULE)
+			finally:
+				sys.path = orig_sys_path
+
+	if template_name not in _django_templates:
+		raise TemplateDoesNotExist()
+
+	run, tmpl, args, optargs = _django_templates[template_name]
+	return (_DjangoTemplayer(run, template_name, tmpl, args, optargs),
+		# make a pretend template name based on the file/fn name
+		run.func_code.co_filename + "." + run.func_code.co_name)
+
+# This oddity is reqired for Django template loaders:
+django_template_loader.is_usable = True
+
+
+def django_template(tmpl, template_name):
+	"""
+	This is a decorator for functions you want to make behave like 
+	standard Django templates.  When django_template_loader is 
+	included in settings.TEMPLATE_LOADERS Django will find the 
+	decorated function when looking for template_name.
+
+	tmpl is the Templayer template object that will be used to create
+	the file layer passed to the decorated function.
+
+	The decorated function must have a signature like:
+	@templayer.django_template(tmpl, "appname/somepage.html")
+	def mock_template(fwriter, context, optional1, optional2 ...)
+	
+	Where fwriter is a new FileWriter object, context is a context
+	instance and all parameters that follow will have their values
+	copied from the context based on the name of the parameters.
+
+	The function must call fwriter.open(..) to render the page then 
+	return the fwriter object (or None) to send it to the client.  
+	fwriter.close() is called automatically.  Other objects returned 
+	or exceptions raised will be passed through to Django.
+	"""
+	global _django_templates
+
+	if not _django_templates:
+		_hook_django_template_loading()
+	
+	def register_mock_template(fn):
+		args, varargs, varkw, defaults = getargspec(fn)
+		if len(args) < 2:
+			raise DjangoTemplayerError("django_template()-"
+				"decorated functions must have fwriter "
+				"and context as their first two parameters")
+
+		nonoptional = len(args) - len(defaults or ())
+		args, optargs = args[2:nonoptional], args[min(2,nonoptional):]
+		_django_templates[template_name] = (fn, tmpl, args, optargs)
+		return fn
+	return register_mock_template
+
+def _hook_django_template_loading():
+	import django.template.loader
+	orig_gtfs = django.template.loader.get_template_from_string
+
+	def get_template_from_string(source, *largs, **dargs):
+		if isinstance(source, _DjangoTemplayer):
+			return source
+		return orig_gtfs(source, *largs, **dargs)
+
+	django.template.loader.get_template_from_string = get_template_from_string
+
+
+def django_view(tmpl):
+	"""
+	This is a decorator for Django view functions that will handle
+	creation of the file layer object from the template tmpl.
+
+	The function must have a signature like:
+	@templayer.django_view(tmpl)
+	def decorated_view(fwriter, request, ...)
+
+	fwriter is the new FileWriter object and the rest of the parameters
+	are the same as what was passed to the view.  
+	
+	The function must call fwriter.open(..) to render the page then 
+	return the fwriter object (or None) to send it to the client.  
+	fwriter.close() is called automatically.  Other objects returned 
+	or exceptions raised will be passed through to Django.
+	"""
+	from django.http import HttpResponse
+	
+	def view_wrapper(fn):
+		def wrapped_view(request, *argl, **argd):
+			# create the file layer before calling our function
+			fwriter = tmpl.start_file(HttpResponse())
+			response = fn(fwriter, request, *argl, **argd)
+			# if the file layer (or None) is returned, render it
+			if response is fwriter or response is None:
+				response = fwriter.close()
+			return response
+
+		# do our best to make the decorated function resemble the 
+		# original function
+		wrapped_view.__name__ = fn.__name__
+		wrapped_view.__dict__.update(fn.__dict__)
+		wrapped_view.__doc__ = fn.__doc__
+		wrapped_view.__module__ = fn.__module__
+		return wrapped_view
+
+	return view_wrapper
+
+
+def get_django_template(name, auto_reload='debug', allow_degenerate=False, 
+	encoding='utf_8'):
+	"""
+	Use Django's template loading machinery to create an
+	HTMLTemplate object.
+
+	auto_reload -- when set to 'debug' will auto-reload only when 
+		settings.DEBUG is set to True
+	"""
+	from django.template.loader import find_template_source
+
+	source, obj = find_template_source(name)
+
+	if auto_reload == 'debug':
+		from django.conf import settings
+		if settings.DEBUG:
+			auto_reload = 0
+		else:
+			auto_reload = 'never'
+
+	tmpl = HTMLTemplate(source, from_string=True, 
+		auto_reload=auto_reload, encoding=encoding)
+	return tmpl
 
 
 if __name__ == "__main__": main()
